require 'test/unit'
require 'digiid'

class TestDigiid < Test::Unit::TestCase

  def setup
    @nonce = "fe32e61882a71074"
    @callback = "http://localhost:3000/callback"
    @uri = "digiid://localhost:3000/callback?x=fe32e61882a71074"
    @address = "1HpE8571PFRwge5coHiFdSCLcwa7qetcn"
    @signature = "IPKm1/EZ1AKscpwSZI34F5NiEkpdr7QKHeLOPPSGs6TXJHULs7CSNtjurcfg72HNuKvL2YgNXdOetQRyARhX7bg="
  end

  def test_build_uri
    digiid = Digiid.new(nonce:@nonce, callback:@callback)

    assert !digiid.uri.nil?
    assert_equal "digiid", digiid.uri.scheme
    assert_equal "localhost", digiid.uri.host
    assert_equal 3000, digiid.uri.port
    assert_equal "/callback", digiid.uri.path

    params = CGI::parse(digiid.uri.query)
    assert_equal @nonce, params['x'].first
  end

  def test_build_qrcode
    digiid = Digiid.new(nonce:@nonce, callback:@callback)

    uri_encoded = CGI::escape(digiid.uri)
    assert_equal "http://chart.apis.google.com/chart?cht=qr&chs=300x300&chl=#{uri_encoded}", digiid.qrcode
  end

  def test_build_uri
    digiid = Digiid.new(nonce:@nonce, callback:@callback)

    assert_match /\Adigiid\:\/\/localhost\:3000\/callback\?x=[a-z0-9]+\Z/, digiid.uri
  end

  def test_build_uri_with_unsecure_param
    digiid = Digiid.new(nonce:@nonce, callback:@callback, unsecure:true)

    assert_match /\Adigiid\:\/\/localhost\:3000\/callback\?x=[a-z0-9]+&u=1\Z/, digiid.uri
  end

  def test_verify_uri
    digiid = Digiid.new(address:@address, uri:@uri, signature:@signature, callback:@callback)
    assert digiid.uri_valid?
  end

  def test_fail_uri_verification_if_bad_uri
    digiid = Digiid.new(address:@address, uri:'garbage', signature:@signature, callback:@callback)
    assert !digiid.uri_valid?
  end

  def test_fail_uri_verification_if_bad_scheme
    digiid = Digiid.new(address:@address, uri:'http://localhost:3000/callback?x=fe32e61882a71074', signature:@signature, callback:@callback)
    assert !digiid.uri_valid?
  end

  def test_fail_uri_verification_if_invalid_callback_url
    digiid = Digiid.new(address:@address, uri:'site.com/callback?x=fe32e61882a71074', signature:@signature, callback:@callback)
    assert !digiid.uri_valid?
  end

  def test_verify_signature
    digiid = Digiid.new(address:@address, uri:@uri, signature:@signature, callback:@callback)
    assert digiid.signature_valid?
  end

  def test_fail_verification_if_invalid_signature
    digiid = Digiid.new(address:@address, uri:@uri, signature:"garbage", callback:@callback)
    assert !digiid.signature_valid?
  end

  def test_fail_verification_if_signature_text_doesnt_match
    digiid = Digiid.new(address:@address, uri:@uri, signature:"H4/hhdnxtXHduvCaA+Vnf0TM4UqdljTsbdIfltwx9+w50gg3mxy8WgLSLIiEjTnxbOPW9sNRzEfjibZXnWEpde4=", callback:@callback)
    assert !digiid.signature_valid?
  end

  def test_extract_nonce
    digiid = Digiid.new(address:@address, uri:@uri, signature:@signature, callback:@callback)
    assert_equal "fe32e61882a71074", digiid.nonce
  end

  def test_testnet
    digiid = Digiid.new(
      address:"mpsaRD2ugdCY1iFrQdsDYRT4qeZzCnvGHW",
      uri:"digiid://digiid.digibyte.blue/callback?x=3893a2a881dd4a1e&u=1",
      signature:"ID5heI0WOeWoryGhZHaxoOH5vkmmcwDsfc4nDQ5vPcXSWh2jyETDGkSNO5zk4nbESGD6k0tgFxYA3HzlEGOf5Uc=",
      callback:"http://digiid.digibyte.blue/callback")
    assert digiid.signature_valid?
  end
end
